@@ -22,8 +22,6 @@ import copy
 import re
 import unicodedata
 
-import six
-
 from cinder.common import constants
 
 
@@ -43,7 +41,7 @@ def _is_printable(char):
 
 def _get_all_chars():
     for i in range(0xFFFF):
-        yield six.unichr(i)
+        yield chr(i)
 
 
 # build a regex that matches all printable characters. This allows
@@ -159,6 +157,15 @@ extra_specs = {
     'additionalProperties': False
 }
 
+image_metadata = {
+    'type': 'object',
+    'patternProperties': {
+        '^[a-zA-Z0-9-_:. /]{1,255}$': {
+            'type': 'string', 'format': 'mysql_text'
+        }
+    },
+    'additionalProperties': False
+}
 
 extra_specs_with_no_spaces_key = {
     'type': 'object',
