@@ -160,6 +160,16 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
             msg = _('nfs volume must be a valid raw or qcow2 image.')
             raise exception.InvalidVolume(reason=msg)
 
+        # Test if the size is accurate or if something tried to modify it
+        if info.virtual_size != volume.size * units.Gi:
+            LOG.error('The volume virtual_size does not match the size in '
+                      'cinder, aborting as we suspect an exploit. '
+                      'Virtual Size is %(vsize)s and real size is %(size)s',
+                      {'vsize': info.virtual_size, 'size': volume.size})
+            msg = _('The volume virtual_size does not match the size in '
+                    'cinder, aborting as we suspect an exploit.')
+            raise exception.InvalidVolume(reason=msg)
+
         conn_info['data']['format'] = info.file_format
         LOG.debug('NfsDriver: conn_info: %s', conn_info)
         return conn_info
@@ -383,18 +393,22 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
             raise exception.ExtendVolumeError(reason='Insufficient space to'
                                               ' extend volume %s to %sG'
                                               % (volume.id, new_size))
-        path = self.local_path(volume)
+        # Use the active image file because this volume might have snapshot(s).
+        active_file = self.get_active_image_from_info(volume)
+        active_file_path = os.path.join(self._local_volume_dir(volume),
+                                        active_file)
         LOG.info('Resizing file to %sG...', new_size)
         file_format = None
         admin_metadata = objects.Volume.get_by_id(
             context.get_admin_context(), volume.id).admin_metadata
+
         if admin_metadata and 'format' in admin_metadata:
             file_format = admin_metadata['format']
-        image_utils.resize_image(path, new_size,
+        image_utils.resize_image(active_file_path, new_size,
                                  run_as_root=self._execute_as_root,
                                  file_format=file_format)
         if file_format == 'qcow2' and not self._is_file_size_equal(
-                path, new_size):
+                active_file_path, new_size):
             raise exception.ExtendVolumeError(
                 reason='Resizing image file failed.')
 
@@ -536,13 +550,13 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
 
         self._stats = data
 
-    @coordination.synchronized('{self.driver_prefix}-{volume[id]}')
+    @coordination.synchronized('{self.driver_prefix}-{volume.id}')
     def create_volume(self, volume):
         """Apply locking to the create volume operation."""
 
         return super(NfsDriver, self).create_volume(volume)
 
-    @coordination.synchronized('{self.driver_prefix}-{volume[id]}')
+    @coordination.synchronized('{self.driver_prefix}-{volume.id}')
     def delete_volume(self, volume):
         """Deletes a logical volume."""
 
@@ -629,6 +643,8 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
         # when this snapshot was created.
         img_info = self._qemu_img_info(forward_path, snapshot.volume.name)
         path_to_snap_img = os.path.join(vol_path, img_info.backing_file)
+        snap_backing_file_img_info = self._qemu_img_info(path_to_snap_img,
+                                                         snapshot.volume.name)
 
         path_to_new_vol = self._local_path_volume(volume)
 
@@ -674,11 +690,13 @@ class NfsDriver(remotefs.RemoteFSSnapDriverDistributed):
                         'luks',
                         passphrase_file=new_pass_file.name,
                         src_passphrase_file=src_pass_file.name,
-                        run_as_root=self._execute_as_root)
+                        run_as_root=self._execute_as_root,
+                        data=snap_backing_file_img_info)
         else:
             image_utils.convert_image(path_to_snap_img,
                                       path_to_new_vol,
                                       out_format,
-                                      run_as_root=self._execute_as_root)
+                                      run_as_root=self._execute_as_root,
+                                      data=snap_backing_file_img_info)
 
         self._set_rw_permissions_for_all(path_to_new_vol)
